# -*- mode: python; python-indent: 2 -*-
#
# xbrightness-rgbgamma - X11 brightness / gamma adjustment tool
#
# Copyright (C) 2011 kakurasan
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# THANKS:
#  xbrightness by Asher Blum and Jean-Daniel Pauget

from gi.repository import Gtk


class GraphDrawingArea (Gtk.DrawingArea):
  """
  DrawingArea for gamma curve
  """
  def __init__ (self, lib):
    Gtk.DrawingArea.__init__ (self)
    self.connect ('configure-event', self._on_self_configure_event)
    self.connect ('draw', self._on_self_draw)
    self._lib = lib
    self._ramp_red = lib.ramp_red
    self._ramp_green = lib.ramp_green
    self._ramp_blue = lib.ramp_blue
    self._ramp_size = lib.ramp_size
  def _on_self_configure_event (self, widget, event):
    """
    'configure-event' handler for this DrawingArea
    """
    self._width = event.width
    self._height = event.height
  def _on_self_draw (self, widget, cr):
    """
    Draw gamma curve
    """
    r = range (self._ramp_size)
    x_step = float (self._width) / float (self._ramp_size - 1)
    y_step = self._height / 256.0
    # bg
    cr.set_source_rgb (1.0, 1.0, 1.0)
    cr.rectangle (0, 0, self._width, self._height)
    cr.fill ()
    # red
    cr.set_source_rgba (0.8, 0.0, 0.0, 0.5)
    cr.new_path ()
    for i in r:
      cr.line_to (x_step * i, y_step * (65535 - self._ramp_red[i]) / 256.0)
    cr.stroke ()
    # green
    cr.set_source_rgba (0.0, 0.8, 0.0, 0.5)
    cr.new_path ()
    for i in r:
      cr.line_to (x_step * i, y_step * (65535 - self._ramp_green[i]) / 256.0)
    cr.stroke ()
    # blue
    cr.set_source_rgba (0.0, 0.0, 0.8, 0.5)
    cr.new_path ()
    for i in r:
      cr.line_to (x_step * i, y_step * (65535 - self._ramp_blue[i]) / 256.0)
    cr.stroke ()
    return False
  def redraw (self):
    """
    Force redraw
    """
    self._ramp_red = self._lib.ramp_red
    self._ramp_green = self._lib.ramp_green
    self._ramp_blue = self._lib.ramp_blue
    self.queue_draw_area (0, 0, self._width, self._height)
