# -*- mode: python; python-indent: 2 -*-
#
# xbrightness-rgbgamma - X11 brightness / gamma adjustment tool
#
# Copyright (C) 2011 kakurasan
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# THANKS:
#  xbrightness by Asher Blum and Jean-Daniel Pauget

from xbrightness_rgbgamma.AppInfo import AppInfo
from xbrightness_rgbgamma.GraphDrawingArea import GraphDrawingArea
import xbrightness_rgbgamma.GuiFunctions as GuiFunctions
import xbrightness_rgbgamma.Libxxf86vm as Libxxf86vm
from gi.repository import Gtk
from gi.repository import GLib
import errno
import os
import gi


class GuiMainWindow (Gtk.Builder):
  """
  Application main window
  """
  _ui = '''
<?xml version="1.0" encoding="UTF-8"?>
<interface>
  <!-- interface-requires gtk+ 3.0 -->
  <object class="GtkAdjustment" id="adj_brightness">
    <property name="upper">65535</property>
    <property name="value">65535</property>
    <property name="step_increment">1</property>
    <property name="page_increment">100</property>
    <signal name="value-changed" handler="_on_adjustment_value_changed" swapped="no"/>
  </object>
  <object class="GtkAdjustment" id="adj_gamma_blue">
    <property name="lower">0.01</property>
    <property name="upper">3</property>
    <property name="value">1</property>
    <property name="step_increment">0.01</property>
    <property name="page_increment">0.050000000000000003</property>
    <signal name="value-changed" handler="_on_adjustment_value_changed" swapped="no"/>
  </object>
  <object class="GtkAdjustment" id="adj_gamma_green">
    <property name="lower">0.01</property>
    <property name="upper">3</property>
    <property name="value">1</property>
    <property name="step_increment">0.01</property>
    <property name="page_increment">0.050000000000000003</property>
    <signal name="value-changed" handler="_on_adjustment_value_changed" swapped="no"/>
  </object>
  <object class="GtkAdjustment" id="adj_gamma_red">
    <property name="lower">0.01</property>
    <property name="upper">3</property>
    <property name="value">1</property>
    <property name="step_increment">0.01</property>
    <property name="page_increment">0.050000000000000003</property>
    <signal name="value-changed" handler="_on_adjustment_value_changed" swapped="no"/>
  </object>
  <object class="GtkWindow" id="window">
    <property name="width_request">400</property>
    <property name="can_focus">False</property>
    <signal name="destroy" handler="_on_window_destroy" swapped="no"/>
    <child>
      <object class="GtkGrid" id="grid_all">
        <property name="visible">True</property>
        <property name="can_focus">False</property>
        <property name="hexpand">True</property>
        <property name="vexpand">True</property>
        <child>
          <object class="GtkMenuBar" id="menubar1">
            <property name="visible">True</property>
            <property name="can_focus">False</property>
            <property name="ubuntu_local">True</property>
            <child>
              <object class="GtkMenuItem" id="item_file">
                <property name="visible">True</property>
                <property name="can_focus">False</property>
                <property name="use_action_appearance">False</property>
                <property name="label" translatable="yes">_File</property>
                <property name="use_underline">True</property>
                <child type="submenu">
                  <object class="GtkMenu" id="menu1">
                    <property name="visible">True</property>
                    <property name="can_focus">False</property>
                    <property name="ubuntu_local">True</property>
                    <child>
                      <object class="GtkCheckMenuItem" id="item_remove_autostart">
                        <property name="visible">True</property>
                        <property name="can_focus">False</property>
                        <property name="use_action_appearance">False</property>
                        <property name="label" translatable="yes">Remove _autostart file on exit</property>
                        <property name="use_underline">True</property>
                      </object>
                    </child>
                    <child>
                      <object class="GtkSeparatorMenuItem" id="separatormenuitem1">
                        <property name="visible">True</property>
                        <property name="can_focus">False</property>
                        <property name="use_action_appearance">False</property>
                      </object>
                    </child>
                    <child>
                      <object class="GtkImageMenuItem" id="item_quit">
                        <property name="label">gtk-quit</property>
                        <property name="visible">True</property>
                        <property name="can_focus">False</property>
                        <property name="use_action_appearance">False</property>
                        <property name="use_underline">True</property>
                        <property name="use_stock">True</property>
                        <accelerator key="q" signal="activate" modifiers="GDK_CONTROL_MASK"/>
                        <signal name="activate" handler="_on_item_quit_activate" swapped="no"/>
                      </object>
                    </child>
                  </object>
                </child>
              </object>
            </child>
            <child>
              <object class="GtkMenuItem" id="item_help">
                <property name="visible">True</property>
                <property name="can_focus">False</property>
                <property name="use_action_appearance">False</property>
                <property name="label" translatable="yes">_Help</property>
                <property name="use_underline">True</property>
                <child type="submenu">
                  <object class="GtkMenu" id="menu3">
                    <property name="visible">True</property>
                    <property name="can_focus">False</property>
                    <property name="ubuntu_local">True</property>
                    <child>
                      <object class="GtkImageMenuItem" id="item_about">
                        <property name="label">gtk-about</property>
                        <property name="visible">True</property>
                        <property name="can_focus">False</property>
                        <property name="use_action_appearance">False</property>
                        <property name="use_underline">True</property>
                        <property name="use_stock">True</property>
                        <signal name="activate" handler="_on_item_about_activate" swapped="no"/>
                      </object>
                    </child>
                  </object>
                </child>
              </object>
            </child>
          </object>
          <packing>
            <property name="left_attach">0</property>
            <property name="top_attach">0</property>
            <property name="width">1</property>
            <property name="height">1</property>
          </packing>
        </child>
        <child>
          <object class="GtkGrid" id="grid_main">
            <property name="visible">True</property>
            <property name="can_focus">False</property>
            <property name="hexpand">True</property>
            <property name="vexpand">True</property>
            <child>
              <object class="GtkGrid" id="grid_controls">
                <property name="visible">True</property>
                <property name="can_focus">False</property>
                <property name="vexpand">True</property>
                <child>
                  <object class="GtkFrame" id="frame1">
                    <property name="visible">True</property>
                    <property name="can_focus">False</property>
                    <property name="label_xalign">0</property>
                    <child>
                      <object class="GtkAlignment" id="alignment1">
                        <property name="visible">True</property>
                        <property name="can_focus">False</property>
                        <property name="left_padding">12</property>
                        <child>
                          <object class="GtkGrid" id="grid_brightness">
                            <property name="visible">True</property>
                            <property name="can_focus">False</property>
                            <child>
                              <object class="GtkScale" id="scale1">
                                <property name="width_request">100</property>
                                <property name="visible">True</property>
                                <property name="can_focus">True</property>
                                <property name="vexpand">True</property>
                                <property name="adjustment">adj_brightness</property>
                                <property name="round_digits">1</property>
                                <property name="draw_value">False</property>
                              </object>
                              <packing>
                                <property name="left_attach">0</property>
                                <property name="top_attach">0</property>
                                <property name="width">1</property>
                                <property name="height">1</property>
                              </packing>
                            </child>
                            <child>
                              <object class="GtkSpinButton" id="spbtn_brightness">
                                <property name="visible">True</property>
                                <property name="can_focus">True</property>
                                <property name="invisible_char">●</property>
                                <property name="adjustment">adj_brightness</property>
                              </object>
                              <packing>
                                <property name="left_attach">1</property>
                                <property name="top_attach">0</property>
                                <property name="width">1</property>
                                <property name="height">1</property>
                              </packing>
                            </child>
                          </object>
                        </child>
                      </object>
                    </child>
                    <child type="label">
                      <object class="GtkLabel" id="label_brightness">
                        <property name="visible">True</property>
                        <property name="can_focus">False</property>
                        <property name="label" translatable="yes">&lt;b&gt;Bright_ness&lt;/b&gt;</property>
                        <property name="use_markup">True</property>
                        <property name="use_underline">True</property>
                        <property name="mnemonic_widget">spbtn_brightness</property>
                      </object>
                    </child>
                  </object>
                  <packing>
                    <property name="left_attach">0</property>
                    <property name="top_attach">0</property>
                    <property name="width">1</property>
                    <property name="height">1</property>
                  </packing>
                </child>
                <child>
                  <object class="GtkFrame" id="frame2">
                    <property name="visible">True</property>
                    <property name="can_focus">False</property>
                    <property name="label_xalign">0</property>
                    <child>
                      <object class="GtkAlignment" id="alignment2">
                        <property name="visible">True</property>
                        <property name="can_focus">False</property>
                        <property name="left_padding">12</property>
                        <child>
                          <object class="GtkGrid" id="grid_gamma">
                            <property name="visible">True</property>
                            <property name="can_focus">False</property>
                            <child>
                              <object class="GtkFrame" id="frame3">
                                <property name="visible">True</property>
                                <property name="can_focus">False</property>
                                <property name="vexpand">True</property>
                                <property name="label_xalign">0</property>
                                <child>
                                  <object class="GtkAlignment" id="alignment3">
                                    <property name="visible">True</property>
                                    <property name="can_focus">False</property>
                                    <property name="left_padding">12</property>
                                    <child>
                                      <object class="GtkGrid" id="grid_gamma_red">
                                        <property name="visible">True</property>
                                        <property name="can_focus">False</property>
                                        <child>
                                          <object class="GtkScale" id="scale2">
                                            <property name="width_request">90</property>
                                            <property name="visible">True</property>
                                            <property name="can_focus">True</property>
                                            <property name="vexpand">True</property>
                                            <property name="adjustment">adj_gamma_red</property>
                                            <property name="round_digits">1</property>
                                            <property name="draw_value">False</property>
                                          </object>
                                          <packing>
                                            <property name="left_attach">0</property>
                                            <property name="top_attach">0</property>
                                            <property name="width">1</property>
                                            <property name="height">1</property>
                                          </packing>
                                        </child>
                                        <child>
                                          <object class="GtkSpinButton" id="spbtn_gamma_red">
                                            <property name="visible">True</property>
                                            <property name="can_focus">True</property>
                                            <property name="invisible_char">●</property>
                                            <property name="adjustment">adj_gamma_red</property>
                                            <property name="digits">2</property>
                                          </object>
                                          <packing>
                                            <property name="left_attach">1</property>
                                            <property name="top_attach">0</property>
                                            <property name="width">1</property>
                                            <property name="height">1</property>
                                          </packing>
                                        </child>
                                      </object>
                                    </child>
                                  </object>
                                </child>
                                <child type="label">
                                  <object class="GtkLabel" id="label6">
                                    <property name="visible">True</property>
                                    <property name="can_focus">False</property>
                                    <property name="label" translatable="yes">&lt;b&gt;_Red&lt;/b&gt;</property>
                                    <property name="use_markup">True</property>
                                    <property name="use_underline">True</property>
                                    <property name="mnemonic_widget">spbtn_gamma_red</property>
                                  </object>
                                </child>
                              </object>
                              <packing>
                                <property name="left_attach">0</property>
                                <property name="top_attach">0</property>
                                <property name="width">1</property>
                                <property name="height">1</property>
                              </packing>
                            </child>
                            <child>
                              <object class="GtkFrame" id="frame4">
                                <property name="visible">True</property>
                                <property name="can_focus">False</property>
                                <property name="vexpand">True</property>
                                <property name="label_xalign">0</property>
                                <child>
                                  <object class="GtkAlignment" id="alignment4">
                                    <property name="visible">True</property>
                                    <property name="can_focus">False</property>
                                    <property name="left_padding">12</property>
                                    <child>
                                      <object class="GtkGrid" id="grid_gamma_green">
                                        <property name="visible">True</property>
                                        <property name="can_focus">False</property>
                                        <child>
                                          <object class="GtkScale" id="scale3">
                                            <property name="width_request">90</property>
                                            <property name="visible">True</property>
                                            <property name="can_focus">True</property>
                                            <property name="vexpand">True</property>
                                            <property name="adjustment">adj_gamma_green</property>
                                            <property name="round_digits">1</property>
                                            <property name="draw_value">False</property>
                                          </object>
                                          <packing>
                                            <property name="left_attach">0</property>
                                            <property name="top_attach">0</property>
                                            <property name="width">1</property>
                                            <property name="height">1</property>
                                          </packing>
                                        </child>
                                        <child>
                                          <object class="GtkSpinButton" id="spbtn_gamma_green">
                                            <property name="visible">True</property>
                                            <property name="can_focus">True</property>
                                            <property name="invisible_char">●</property>
                                            <property name="adjustment">adj_gamma_green</property>
                                            <property name="digits">2</property>
                                          </object>
                                          <packing>
                                            <property name="left_attach">1</property>
                                            <property name="top_attach">0</property>
                                            <property name="width">1</property>
                                            <property name="height">1</property>
                                          </packing>
                                        </child>
                                      </object>
                                    </child>
                                  </object>
                                </child>
                                <child type="label">
                                  <object class="GtkLabel" id="label7">
                                    <property name="visible">True</property>
                                    <property name="can_focus">False</property>
                                    <property name="label" translatable="yes">&lt;b&gt;_Green&lt;/b&gt;</property>
                                    <property name="use_markup">True</property>
                                    <property name="use_underline">True</property>
                                    <property name="mnemonic_widget">spbtn_gamma_green</property>
                                  </object>
                                </child>
                              </object>
                              <packing>
                                <property name="left_attach">0</property>
                                <property name="top_attach">1</property>
                                <property name="width">1</property>
                                <property name="height">1</property>
                              </packing>
                            </child>
                            <child>
                              <object class="GtkFrame" id="frame5">
                                <property name="visible">True</property>
                                <property name="can_focus">False</property>
                                <property name="vexpand">True</property>
                                <property name="label_xalign">0</property>
                                <child>
                                  <object class="GtkAlignment" id="alignment5">
                                    <property name="visible">True</property>
                                    <property name="can_focus">False</property>
                                    <property name="left_padding">12</property>
                                    <child>
                                      <object class="GtkGrid" id="grid_gamma_blue">
                                        <property name="visible">True</property>
                                        <property name="can_focus">False</property>
                                        <child>
                                          <object class="GtkScale" id="scale4">
                                            <property name="width_request">90</property>
                                            <property name="visible">True</property>
                                            <property name="can_focus">True</property>
                                            <property name="vexpand">True</property>
                                            <property name="adjustment">adj_gamma_blue</property>
                                            <property name="round_digits">1</property>
                                            <property name="draw_value">False</property>
                                          </object>
                                          <packing>
                                            <property name="left_attach">0</property>
                                            <property name="top_attach">0</property>
                                            <property name="width">1</property>
                                            <property name="height">1</property>
                                          </packing>
                                        </child>
                                        <child>
                                          <object class="GtkSpinButton" id="spbtn_gamma_blue">
                                            <property name="visible">True</property>
                                            <property name="can_focus">True</property>
                                            <property name="invisible_char">●</property>
                                            <property name="adjustment">adj_gamma_blue</property>
                                            <property name="digits">2</property>
                                          </object>
                                          <packing>
                                            <property name="left_attach">1</property>
                                            <property name="top_attach">0</property>
                                            <property name="width">1</property>
                                            <property name="height">1</property>
                                          </packing>
                                        </child>
                                      </object>
                                    </child>
                                  </object>
                                </child>
                                <child type="label">
                                  <object class="GtkLabel" id="label8">
                                    <property name="visible">True</property>
                                    <property name="can_focus">False</property>
                                    <property name="label" translatable="yes">&lt;b&gt;_Blue&lt;/b&gt;</property>
                                    <property name="use_markup">True</property>
                                    <property name="use_underline">True</property>
                                    <property name="mnemonic_widget">spbtn_gamma_blue</property>
                                  </object>
                                </child>
                              </object>
                              <packing>
                                <property name="left_attach">0</property>
                                <property name="top_attach">2</property>
                                <property name="width">1</property>
                                <property name="height">1</property>
                              </packing>
                            </child>
                          </object>
                        </child>
                      </object>
                    </child>
                    <child type="label">
                      <object class="GtkLabel" id="label_gamma">
                        <property name="visible">True</property>
                        <property name="can_focus">False</property>
                        <property name="label" translatable="yes">&lt;b&gt;Gamma correction&lt;/b&gt;</property>
                        <property name="use_markup">True</property>
                      </object>
                    </child>
                  </object>
                  <packing>
                    <property name="left_attach">0</property>
                    <property name="top_attach">1</property>
                    <property name="width">1</property>
                    <property name="height">1</property>
                  </packing>
                </child>
                <child>
                  <object class="GtkButton" id="button_apply">
                    <property name="label">gtk-apply</property>
                    <property name="visible">True</property>
                    <property name="sensitive">False</property>
                    <property name="can_focus">True</property>
                    <property name="receives_default">True</property>
                    <property name="use_action_appearance">False</property>
                    <property name="use_stock">True</property>
                    <signal name="clicked" handler="_on_button_apply_clicked" swapped="no"/>
                  </object>
                  <packing>
                    <property name="left_attach">0</property>
                    <property name="top_attach">2</property>
                    <property name="width">1</property>
                    <property name="height">1</property>
                  </packing>
                </child>
              </object>
              <packing>
                <property name="left_attach">1</property>
                <property name="top_attach">0</property>
                <property name="width">1</property>
                <property name="height">1</property>
              </packing>
            </child>
            <child>
              <placeholder/>
            </child>
          </object>
          <packing>
            <property name="left_attach">0</property>
            <property name="top_attach">1</property>
            <property name="width">1</property>
            <property name="height">1</property>
          </packing>
        </child>
        <child>
          <object class="GtkStatusbar" id="statusbar">
            <property name="visible">True</property>
            <property name="can_focus">False</property>
            <property name="orientation">vertical</property>
            <property name="spacing">2</property>
          </object>
          <packing>
            <property name="left_attach">0</property>
            <property name="top_attach">2</property>
            <property name="width">1</property>
            <property name="height">1</property>
          </packing>
        </child>
      </object>
    </child>
  </object>
</interface>
'''
  _toid = None
  _desktopfile = os.path.join (GLib.get_user_config_dir (), 'autostart', 'xbrightness-rgbgamma.desktop')
  def __init__ (self, lib):
    Gtk.Builder.__init__ (self)
    self.add_from_string (self._ui)
    # GtkBuilder translation fix
    for obj in self.get_objects ():
      if obj.__gtype__ == gi.repository.Gtk.Label.__gtype__ or \
       obj.__gtype__ == gi.repository.Gtk.MenuItem.__gtype__ or \
       obj.__gtype__ == gi.repository.Gtk.CheckMenuItem.__gtype__:
        obj.set_label (_(obj.get_label ()))
    # lib
    self._lib = lib
    self._brightness = self._lib.brightness
    self._gamma_red = self._lib.gamma_red
    self._gamma_green = self._lib.gamma_green
    self._gamma_blue = self._lib.gamma_blue
    # graph
    self._drawingarea = GraphDrawingArea (self._lib)
    # adjustments
    self._adjustment_brightness = self.get_object ('adj_brightness')
    self._adjustment_brightness.props.value = self._brightness
    self._adjustment_gamma_red = self.get_object ('adj_gamma_red')
    self._adjustment_gamma_red.props.value = self._gamma_red
    self._adjustment_gamma_green = self.get_object ('adj_gamma_green')
    self._adjustment_gamma_green.props.value = self._gamma_green
    self._adjustment_gamma_blue = self.get_object ('adj_gamma_blue')
    self._adjustment_gamma_blue.props.value = self._gamma_blue
    # signal
    self.connect_signals (self)
    # 'remove autostart' menu item
    self._item_remove_autostart = self.get_object ('item_remove_autostart')
    # statusbar
    self._statusbar = self.get_object ('statusbar')
    self._ctxid_status = self._statusbar.get_context_id ('status')
    self._ctxid_count = self._statusbar.get_context_id ('count')
    # 'apply' button
    self._button_apply = self.get_object ('button_apply')
    # gamma chart
    notebook_gamma = Gtk.Notebook ()
    notebook_gamma.append_page (Gtk.Image (file=os.path.join (AppInfo.prefix, 'share', AppInfo.pkgname, 'gammachart', '2.2.png')), Gtk.Label (label='2._2', use_underline=True))
    notebook_gamma.append_page (Gtk.Image (file=os.path.join (AppInfo.prefix, 'share', AppInfo.pkgname, 'gammachart', '2.0.png')), Gtk.Label (label='2._0', use_underline=True))
    notebook_gamma.append_page (Gtk.Image (file=os.path.join (AppInfo.prefix, 'share', AppInfo.pkgname, 'gammachart', '1.8.png')), Gtk.Label (label='1._8', use_underline=True))
    notebook_gamma.append_page (Gtk.Image (file=os.path.join (AppInfo.prefix, 'share', AppInfo.pkgname, 'gammachart', '1.0.png')), Gtk.Label (label='_1.0', use_underline=True))
    # layout
    aspectframe = Gtk.AspectFrame (yalign=0)
    aspectframe.add (self._drawingarea)
    notebook_graph_chart = Gtk.Notebook ()
    notebook_graph_chart.props.expand = notebook_graph_chart.props.hexpand = True
    notebook_graph_chart.append_page (aspectframe, Gtk.Label (label=_('Gra_ph'), use_underline=True))
    notebook_graph_chart.append_page (notebook_gamma, Gtk.Label (label=_('Gamma _chart'), use_underline=True))
    self.get_object ('grid_main').attach (notebook_graph_chart, 0, 0, 1, 1)
    # window
    self._window = self.get_object ('window')
    self._window.props.title = AppInfo.name
    self._window.props.icon = AppInfo.icon
  def _on_item_quit_activate (self, item):
    """
    'activate' handler for 'quit' menu item
    """
    self._on_quit ()
  def _on_item_about_activate (self, item):
    """
    'activate' handler for 'about' menu item
    """
    GuiFunctions.run_aboutdialog ()
  def _on_window_destroy (self, widget):
    """
    'destroy' handler for main window
    """
    self._on_quit ()
  def _on_adjustment_value_changed (self, adjustment):
    """
    'value-changed' handler for brightness/gamma
    """
    try:
      self._lib.set_gamma_and_brightness (self._adjustment_brightness.props.value,
                                          self._adjustment_gamma_red.props.value,
                                          self._adjustment_gamma_green.props.value,
                                          self._adjustment_gamma_blue.props.value)
      self._drawingarea.redraw ()
      self._statusbar.pop (self._ctxid_status)
      if adjustment == self._adjustment_brightness:
        self._statusbar.push (self._ctxid_status, _('Set brightness to {0:d}').format (int (adjustment.props.value)))
      elif adjustment == self._adjustment_gamma_red:
        self._statusbar.push (self._ctxid_status, _('Set gamma/red to {0:.2f}').format (adjustment.props.value))
      elif adjustment == self._adjustment_gamma_green:
        self._statusbar.push (self._ctxid_status, _('Set gamma/green to {0:.2f}').format (adjustment.props.value))
      elif adjustment == self._adjustment_gamma_blue:
        self._statusbar.push (self._ctxid_status, _('Set gamma/blue to {0:.2f}').format (adjustment.props.value))
      if not self._button_apply.props.sensitive:
        self._button_apply.props.sensitive = True
        self._cnt = 10
        self._toid = GLib.timeout_add_seconds (1, self._to_revert_countdown)
    except Libxxf86vm.SetGammaRampError:
      GuiFunctions.run_errordialog (self, _('Could not set gamma ramp.'))
  def _to_revert_countdown (self):
    """
    Count down "cancel after N sec(s)" counter
    """
    self._cnt -= 1
    self._statusbar.pop (self._ctxid_count)
    self._statusbar.push (self._ctxid_count, ngettext ('Cancel after {0} second', 'Cancel after {0} seconds', self._cnt).format (self._cnt))
    if self._cnt > 0:
      return True  # continue
    else:
      # revert: lib
      try:
        self._lib.set_gamma_and_brightness (self._brightness,
                                            self._gamma_red,
                                            self._gamma_green,
                                            self._gamma_blue)
        self._drawingarea.redraw ()
        # revert: widgets
        self._adjustment_brightness.props.value = self._brightness
        self._adjustment_gamma_red.props.value = self._gamma_red
        self._adjustment_gamma_green.props.value = self._gamma_green
        self._adjustment_gamma_blue.props.value = self._gamma_blue
        self._statusbar.pop (self._ctxid_status)
        self._statusbar.push (self._ctxid_status, _('Values are reverted').format (self._cnt))
        self._button_apply.props.sensitive = False
      except Libxxf86vm.SetGammaRampError:
        GuiFunctions.run_errordialog (self, _('Could not set gamma ramp.'))
      self._toid = None
      return False  # stop
  def _on_button_apply_clicked (self, widget):
    """
    Apply values
    """
    GLib.source_remove (self._toid)
    self._toid = None
    self._brightness = self._adjustment_brightness.props.value
    self._gamma_red = self._adjustment_gamma_red.props.value
    self._gamma_green = self._adjustment_gamma_green.props.value
    self._gamma_blue = self._adjustment_gamma_blue.props.value
    self._button_apply.props.sensitive = False
    self._statusbar.pop (self._ctxid_status)
    self._statusbar.push (self._ctxid_status, _('Values are applied').format (self._cnt))
    # autostart
    if os.access (os.path.join (os.path.dirname (AppInfo.prefix), 'exec-cli.py'), os.X_OK):
      # source (not installed)
      cmd = os.path.join (os.path.dirname (AppInfo.prefix), 'exec-cli.py')
    else:
      # system (installed)
      cmd = 'xbrightness-rgbgamma'
    try:
      with open (self._desktopfile, 'w') as f_out:
        try:
          f_out.write ('''[Desktop Entry]
Type=Application
Exec={cmd} {brightness} {gamma_red} {gamma_green} {gamma_blue}
Hidden=false
NoDisplay=false
X-GNOME-Autostart-enabled=true
Name=xbrightness-rgbgamma
Comment=Brightness and gamma
'''.format (cmd=cmd,
            brightness=int (self._brightness),
            gamma_red=self._gamma_red,
            gamma_green=self._gamma_green,
            gamma_blue=self._gamma_blue))
        except IOError as e:
          GuiFunctions.run_errordialog (self, _('Could not write to file "{0}"').format (self._desktopfile), e)
    except IOError as e:
      GuiFunctions.run_errordialog (self, _('Could not open file "{0}"').format (self._desktopfile), e)
  def _on_quit (self):
    """
    Quit
    """
    if self._toid:
      # if not applied, revert
      try:
        self._lib.set_gamma_and_brightness (self._brightness,
                                            self._gamma_red,
                                            self._gamma_green,
                                            self._gamma_blue)
      except Libxxf86vm.SetGammaRampError:
        pass
    if self._item_remove_autostart.props.active == True:
      # remove .desktop file
      try:
        os.unlink (self._desktopfile)
      except OSError as e:
        if e.errno != errno.ENOENT:
          GuiFunctions.run_errordialog (self, _('Could not remove file "{0}"').format (self._desktopfile), e)
    Gtk.main_quit ()
  def show_all (self):
    """
    Show main window and its childs
    """
    self._window.show_all ()
